# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
from typing import List, Union
from PySide6 import QtCore, QtGui
from VeraGrid.Gui.Diagrams.SchematicWidget.schematic_widget import SchematicWidget
from VeraGrid.Gui.Diagrams.MapWidget.grid_map_widget import GridMapWidget


class DiagramsModel(QtCore.QAbstractListModel):
    """
    Model for the diagrams
    # from VeraGrid.Gui.Diagrams.BusViewer.bus_viewer_dialogue import BusViewerGUI
    # from VeraGrid.Gui.DiagramEditorWidget import DiagramEditorWidget
    # from VeraGrid.Gui.Diagrams.MapWidget.grid_map_widget import GridMapWidget
    """

    def __init__(self, list_of_diagrams: List[Union[SchematicWidget, GridMapWidget]]):
        """
        Enumeration model
        :param list_of_diagrams: list of enumeration values to show
        """
        QtCore.QAbstractListModel.__init__(self)
        self.items = list_of_diagrams

        self.bus_branch_editor_icon = QtGui.QIcon()
        self.bus_branch_editor_icon.addPixmap(QtGui.QPixmap(":/Icons/icons/schematic.png"))

        self.bus_branch_vicinity_icon = QtGui.QIcon()
        self.bus_branch_vicinity_icon.addPixmap(QtGui.QPixmap(":/Icons/icons/grid_icon.png"))

        self.map_editor_icon = QtGui.QIcon()
        self.map_editor_icon.addPixmap(QtGui.QPixmap(":/Icons/icons/map.png"))

    def flags(self, index: QtCore.QModelIndex):
        """
        Get the display mode
        :param index:
        :return:
        """
        return (QtCore.Qt.ItemFlag.ItemIsEditable |
                QtCore.Qt.ItemFlag.ItemIsEnabled |
                QtCore.Qt.ItemFlag.ItemIsSelectable)

    def rowCount(self, parent=QtCore.QModelIndex()) -> int:
        """

        :param parent:
        :return:
        """
        return len(self.items)

    def data(self, index: QtCore.QModelIndex, role=QtCore.Qt.ItemDataRole.DisplayRole):
        """

        :param index:
        :param role:
        :return:
        """
        if index.isValid():

            if index.row() < len(self.items):
                diagram = self.items[index.row()]

                if (role == QtCore.Qt.ItemDataRole.DisplayRole or
                        role == QtCore.Qt.ItemDataRole.EditRole):
                    return diagram.name

                elif role == QtCore.Qt.ItemDataRole.DecorationRole:
                    if isinstance(diagram, SchematicWidget):
                        return self.bus_branch_editor_icon
                    elif isinstance(diagram, GridMapWidget):
                        return self.map_editor_icon

        return None

    def setData(self, index, value, role=None):
        """
        Set data by simple editor (whatever text)
        :param index:
        :param value:
        :param role:
        :return:
        """

        self.items[index.row()].name = value
        self.dataChanged.emit(index, index, [role])  # keep view in sync
        return True



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations
from typing import TYPE_CHECKING
import os
from PySide6 import QtWidgets
from VeraGrid.Gui.PsseDialogue.psse_export_gui import Ui_PsseExportDialog
from VeraGrid.Gui.general_dialogues import LogsDialogue
import VeraGrid.Session.file_handler as filedrv
from VeraGridEngine.basic_structures import Logger
from VeraGridEngine.enumerations import FileType

if TYPE_CHECKING:
    from VeraGrid.Gui.Main.SubClasses.io import IoMain


class PsseExportDialogue(QtWidgets.QDialog):
    """
    GridMergeDialogue
    """

    def __init__(self, app: IoMain):
        """
        GridMergeDialogue
        :param App: App pointer
        """
        QtWidgets.QDialog.__init__(self)
        self.ui = Ui_PsseExportDialog()
        self.ui.setupUi(self)
        self.setWindowTitle('PSS/e export')
        self.setModal(True)

        self.app = app

        self.logger = Logger()
        self.logs_dialogue: LogsDialogue | None = None

        self.ui.raw_export_version_comboBox.addItems(["33", "35"])

        self.ui.exportButton.clicked.connect(self.export)

    def export(self):
        """

        :return:
        """
        raw_version = self.ui.raw_export_version_comboBox.currentText()

        # if the global file_name is empty, ask where to save
        fname = os.path.join(self.app.project_directory, self.app.ui.grid_name_line_edit.text())

        files_types = "PSS/e raw (*.raw);;PSS/e rawx (*.rawx)"
        filename, type_selected = QtWidgets.QFileDialog.getSaveFileName(self,
                                                                        'Export to PSS/e',
                                                                        fname,
                                                                        files_types)

        if filename != '':

            # if the user did not enter the extension, add it automatically
            name, file_extension = os.path.splitext(filename)

            if filename.endswith(".raw"):
                file_type = FileType.PSSE_raw
            elif filename.endswith(".rawx"):
                file_type = FileType.PSSE_rawx
            else:
                file_type = FileType.PSSE_raw

            if file_extension == '':
                filename = name + '.raw'
                file_type = FileType.PSSE_raw

            # we were able to compose the file correctly, now save it
            self.app.save_file_now(
                filename=filename,
                type_selected=type_selected,
                options=filedrv.FileSavingOptions(
                    raw_version=raw_version,
                    file_type=file_type
                )
            )

            self.app.show_info_toast("PSS/e export done!")
            self.close()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
import json
import os
from typing import Dict, Union, Any
from PySide6 import QtWidgets

import VeraGrid.ThirdParty.qdarktheme as qdarktheme
from VeraGridEngine.IO.file_system import get_create_veragrid_folder
from VeraGrid.Gui.Main.SubClasses.Results.results import ResultsMain
from VeraGrid.Gui.Diagrams.SchematicWidget.schematic_widget import SchematicWidget
from VeraGrid.Gui.Diagrams.generic_graphics import set_dark_mode, set_light_mode
from VeraGrid.plugins import PluginsInfo, PluginFunction
from VeraGrid.Gui.gui_functions import add_menu_entry


def gui_struct_to_data(data_: Dict[str, Union[float, int, str, bool, Dict[str, Union[float, int, str, bool, Dict]]]],
                       struct_: Dict[str, Dict[str, any]]):
    """
    Recursive function to get the config dictionary from the GUI values
    :param data_: Dictionary to fill
    :param struct_: the result with self.get_config_structure()
    """
    for key, value in struct_.items():
        if isinstance(value, dict):
            data_[key] = dict()
            gui_struct_to_data(data_[key], value)
        elif isinstance(value, QtWidgets.QComboBox):
            data_[key] = value.currentText()
        elif isinstance(value, QtWidgets.QDoubleSpinBox):
            data_[key] = value.value()
        elif isinstance(value, QtWidgets.QSpinBox):
            data_[key] = value.value()
        elif isinstance(value, QtWidgets.QCheckBox):
            data_[key] = value.isChecked()
        elif isinstance(value, QtWidgets.QRadioButton):
            data_[key] = value.isChecked()
        elif isinstance(value, str):
            data_[key] = value
        elif isinstance(value, int):
            data_[key] = value
        elif isinstance(value, float):
            data_[key] = value
        elif isinstance(value, bool):
            data_[key] = value
        else:
            raise Exception(f'unknown structure {value}')


def config_data_to_struct(data_: Dict[str, Union[Dict[str, Any], str, Any]],
                          struct_: Dict[str, Dict[str, Any]]) -> None:
    """
    Recursive function to set the GUI objects' values from the config dictionary
    :param data_: config dictionary with values from the file
    :param struct_: result of self.get_config_structure()
    """
    for key, object_to_set in struct_.items():

        # get the value in data_ that corresponds to the object to be set
        corresponding_data = data_.get(key, None)

        if corresponding_data is not None:

            # print("config debug:", key, corresponding_data)

            if isinstance(object_to_set, dict):
                config_data_to_struct(corresponding_data, object_to_set)

            elif isinstance(object_to_set, QtWidgets.QComboBox):
                index = object_to_set.findText(str(corresponding_data))
                if -1 < index < object_to_set.count():
                    object_to_set.setCurrentIndex(index)

            elif isinstance(object_to_set, QtWidgets.QDoubleSpinBox):
                object_to_set.setValue(float(corresponding_data))

            elif isinstance(object_to_set, QtWidgets.QSpinBox):
                object_to_set.setValue(int(corresponding_data))

            elif isinstance(object_to_set, QtWidgets.QCheckBox):
                object_to_set.setChecked(bool(corresponding_data))

            elif isinstance(object_to_set, QtWidgets.QRadioButton):
                object_to_set.setChecked(bool(corresponding_data))

            elif isinstance(object_to_set, str):
                pass
            elif isinstance(object_to_set, float):
                pass
            elif isinstance(object_to_set, int):
                pass
            elif isinstance(object_to_set, bool):
                pass
            else:
                raise Exception('unknown structure')
        else:
            print(f"{key} has no entry in config")


class ConfigurationMain(ResultsMain):
    """
    Diagrams Main
    """

    def __init__(self, parent=None):
        """

        @param parent:
        """

        # create main window
        ResultsMain.__init__(self, parent)

        # plugins
        self.plugins_info = PluginsInfo()

        # check boxes
        self.ui.dark_mode_checkBox.clicked.connect(self.change_theme_mode)

        self.plugins_investment_evaluation_method_dict = dict()

        # DateTime change
        self.ui.snapshot_dateTimeEdit.dateTimeChanged.connect(self.snapshot_datetime_changed)

        self.plugin_windows_list = list()

    def change_theme_mode(self) -> None:
        """
        Change the GUI theme
        """
        custom_colors = {"primary": "#00aa88ff",
                         "primary>list.selectionBackground": "#00aa88be"}

        if self.ui.dark_mode_checkBox.isChecked():
            set_dark_mode()

            qdarktheme.setup_theme(theme='dark',
                                   custom_colors=custom_colors,
                                   additional_qss="QToolTip {color: white; background-color: black; border: 0px; }")

            # note: The 0px border on the tooltips allow it to render properly
            for diagram in self.diagram_widgets_list:
                if isinstance(diagram, SchematicWidget):
                    diagram.set_dark_mode()

            self.colour_diagrams()

        else:
            set_light_mode()
            qdarktheme.setup_theme(theme='light',
                                   custom_colors=custom_colors,
                                   additional_qss="QToolTip {color: black; background-color: white; border: 0px;}")

            # note: The 0px border on the tooltips allow it to render properly
            for diagram in self.diagram_widgets_list:
                if isinstance(diagram, SchematicWidget):
                    diagram.set_light_mode()

            self.colour_diagrams()

    @staticmethod
    def config_file_path() -> str:
        """
        get the config file path
        :return: config file path
        """
        return os.path.join(get_create_veragrid_folder(), 'gui_config.json')

    def config_file_exists(self) -> bool:
        """
        Check if the config file exists
        :return: True / False
        """
        return os.path.exists(self.config_file_path())

    def get_config_structure(self) -> Dict[str, Dict[str, any]]:
        """
        Get the settings configuration dictionary
        This serves to collect automatically the settings
        and apply the incomming setting automatically as well
        :return: Dict[name, Dict[name, QtWidget]
        """
        return {
            "graphics": {
                "dark_mode": self.ui.dark_mode_checkBox,
                "palette": self.ui.palette_comboBox,
                # "min_node_size": self.ui.min_node_size_spinBox,
                # "max_node_size": self.ui.max_node_size_spinBox,
                # "min_branch_size": self.ui.min_branch_size_spinBox,
                # "max_branch_size": self.ui.max_branch_size_spinBox,
                # "width_based_flow": self.ui.branch_width_based_on_flow_checkBox,
                "map_tile_provider": self.ui.tile_provider_comboBox,
                "plotting_style": self.ui.plt_style_comboBox,
                "video_fps": self.ui.fps_spinBox
            },
            "machine_learning": {
                "clustering": {
                    "cluster_number": self.ui.cluster_number_spinBox,
                },
                "node_grouping": {
                    "sigma": self.ui.node_distances_sigma_doubleSpinBox,
                    "n_elements": self.ui.node_distances_elements_spinBox,
                },
                "investments_evaluation": {
                    "investment_evaluation_method": self.ui.investment_evaluation_method_ComboBox,
                    "max_investments_evluation_number": self.ui.max_investments_evluation_number_spinBox,
                    "investment_evaluation_obj_func": self.ui.investment_evaluation_objfunc_ComboBox,
                    "firm_capacity_share": self.ui.firmCapacityShareSpinBox,
                },
                "stochastic": {
                    "method": self.ui.stochastic_pf_method_comboBox,
                    "voltage_variance": self.ui.tolerance_stochastic_spinBox,
                    "number_of_samples": self.ui.max_iterations_stochastic_spinBox
                },
                "cascading": {
                    "additional_islands": self.ui.cascading_islands_spinBox
                },
                "reliability": {
                    "method": self.ui.reliability_method_comboBox,
                    "number_of_samples": self.ui.reliability_method_comboBox
                },
            },
            "linear": {
                "ptdf_threshold": self.ui.ptdf_threshold_doubleSpinBox,
                "lodf_threshold": self.ui.lodf_threshold_doubleSpinBox
            },
            "power_flow": {
                "solver": self.ui.solver_comboBox,
                "retry": self.ui.helm_retry_checkBox,
                "three_phase": self.ui.pf_three_phase_checkBox,
                "distributed_slack": self.ui.distributed_slack_checkBox,
                "ignore_single_node_islands": self.ui.ignore_single_node_islands_checkBox,
                "use_voltage_guess": self.ui.use_voltage_guess_checkBox,
                "precision": self.ui.tolerance_spinBox,
                "acceleration": self.ui.muSpinBox,
                "max_iterations": self.ui.max_iterations_spinBox,
                "verbosity": self.ui.verbositySpinBox,

                "reactive_power_control_mode": self.ui.control_q_checkBox,
                "transformer_taps_module_control": self.ui.control_tap_modules_checkBox,
                "transformer_taps_phase_control": self.ui.control_tap_phase_checkBox,
                "remote_voltage_controls_switch": self.ui.control_remote_voltage_checkBox,
                "orthogonalize_controls": self.ui.orthogonalize_pf_controls_checkBox,

                "apply_temperature_correction": self.ui.temperature_correction_checkBox,
                "apply_impedance_tolerances": self.ui.apply_impedance_tolerances_checkBox,
                "add_pf_report": self.ui.addPowerFlowReportCheckBox,
                "initialize_angles": self.ui.initialize_pf_angles_checkBox,
            },
            "state_estimation": {
                "solver": self.ui.se_solver_comboBox,
                "tolerance": self.ui.se_tolerance_spinBox,
                "max_iter": self.ui.se_max_iterations_spinBox,
                "prefer_correct": self.ui.se_prefer_correct_checkBox,
                "fixed_slack": self.ui.se_fixed_slack_checkBox,
                "run_observability_analyis": self.ui.se_observability_analysis_checkBox,
                "add_pseudo_measurements": self.ui.se_add_pseudo_measurements_checkBox,
                "run_measurement_profiling": self.ui.se_measurements_profiling_checkBox
            },
            "optimal_power_flow": {
                "method": self.ui.lpf_solver_comboBox,
                "time_grouping": self.ui.opf_time_grouping_comboBox,
                "zone_grouping": self.ui.opfZonalGroupByComboBox,
                "mip_solver": self.ui.mip_solver_comboBox,
                "contingency_tolerance": self.ui.opfContingencyToleranceSpinBox,
                "skip_generation_limits": self.ui.skipOpfGenerationLimitsCheckBox,
                "consider_contingencies": self.ui.considerContingenciesOpfCheckBox,
                "dispatch_mode": self.ui.opfDispatchModeComboBox,
                "consider_ramps": self.ui.opfConsiderRampsCheckBox,
                "consider_time_up_down": self.ui.opfConsiderUpDownTimeCheckBox,
                "area_spinning_reserve": self.ui.opfSpinningReserveCheckBox,
                "add_opf_report": self.ui.addOptimalPowerFlowReportCheckBox,
                "robust_mip": self.ui.fixOpfCheckBox,
                "save_mip": self.ui.save_mip_checkBox,
                "use_glsk_as_cost": self.ui.useGslkAsCostsOpfCheckBox,
                "add_losses_approximation": self.ui.approximateLossesOpfCheckBox,
                "ips_method": self.ui.ips_method_comboBox,
                "ips_tolerance": self.ui.ips_tolerance_spinBox,
                "ips_iterations": self.ui.ips_iterations_spinBox,
                "ips_trust_radius": self.ui.ips_trust_radius_doubleSpinBox,
                "ips_init_with_pf": self.ui.ips_initialize_with_pf_checkBox,
                "ips_control_Qlimits": self.ui.ips_control_Qlimits_checkBox,
            },
            "continuation_power_flow": {
                "max_iterations": self.ui.vs_max_iterations_spinBox,
                "stop_at": self.ui.vc_stop_at_comboBox,
                "increase_system_loading": self.ui.start_vs_from_default_radioButton,
                "lambda_factor": self.ui.alpha_doubleSpinBox,
                "points_from_time_series": self.ui.start_vs_from_selected_radioButton,
                "now": self.ui.vs_departure_comboBox,
                "target": self.ui.vs_target_comboBox,
                "available_transfer_capacity": self.ui.atcRadioButton,
            },
            "net_transfer_capacity": {
                "transfer_sensitivity_threshold": self.ui.atcThresholdSpinBox,
                "transfer_method": self.ui.transferMethodComboBox,
                "Loading_threshold_to_report": self.ui.ntcReportLoadingThresholdSpinBox,
                "ntc_linear_consider_contingencies": self.ui.n1ConsiderationCheckBox,

                "skip_generation_limits": self.ui.skipNtcGenerationLimitsCheckBox,
                "transmission_reliability_margin": self.ui.trmSpinBox,

                "use_branch_exchange_sensitivity": self.ui.ntcSelectBasedOnExchangeSensitivityCheckBox,
                "branch_exchange_sensitivity": self.ui.ntcAlphaSpinBox,

                "use_branch_rating_contribution": self.ui.ntcSelectBasedOnAcerCriteriaCheckBox,
                "branch_rating_contribution": self.ui.ntcLoadRuleSpinBox,

                "ntc_opt_consider_contingencies": self.ui.consider_ntc_contingencies_checkBox,
            },
            "nodal_hosting_capacity": {
                "method": self.ui.nodal_capacity_method_comboBox,
                "sense": self.ui.nodal_capacity_sense_SpinBox
            },
            "general": {
                "base_power": self.ui.sbase_doubleSpinBox,
                "frequency": self.ui.fbase_doubleSpinBox,
                "default_bus_voltage": self.ui.defaultBusVoltageSpinBox,
                "engine": self.ui.engineComboBox
            },
            "contingencies": {
                "contingencies_engine": self.ui.contingencyEngineComboBox,
                "use_srap": self.ui.use_srap_checkBox,
                "srap_max_power": self.ui.srap_limit_doubleSpinBox,
                "srap_top_n": self.ui.srap_top_n_SpinBox,
                "srap_deadband": self.ui.srap_deadband_doubleSpinBox,
                "contingency_deadband": self.ui.contingency_deadband_SpinBox,
                "srap_revert_to_nominal_rating": self.ui.srap_revert_to_nominal_rating_checkBox,
                "contingency_massive_report": self.ui.contingency_detailed_massive_report_checkBox
            },
            "file": {
                "store_results_in_file": self.ui.saveResultsCheckBox,
                # "current_boundary_set": self.current_boundary_set,
                # "cgmes_selected_version": self.ui.cgmes_version_comboBox,
                # "cgmes_single_profile_per_file": self.ui.cgmes_single_profile_per_file_checkBox,
                # "map_regions_like_raw": self.ui.cgmes_map_regions_like_raw_checkBox,
                # "raw_selected_version": self.ui.raw_export_version_comboBox,
                # "cgmes_dc_as_hvdclines": self.ui.cgmes_dc_as_hvdclines_checkBox,
            },
            "dyn": {
                "rms_int_method_comboBox": self.ui.rms_integration_method_comboBox,
                "rms_initialization_method": self.ui.rms_initialization_method_comboBox,
                "tolerance_rms_spinBox": self.ui.tolerance_rms_spinBox,
                "sim_time_spinBox": self.ui.sim_time_spinBox,
                "h_spinBox": self.ui.h_spinBox,

                "ss_assessment_time_spinBox_2": self.ui.ss_assessment_time_spinBox_2
            }
        }

    def get_gui_config_data(self) -> Dict[str, Dict[str, Union[float, int, str, bool]]]:
        """
        Get a dictionary with the GUI configuration data
        :return:
        """
        struct = self.get_config_structure()
        data = dict()
        gui_struct_to_data(data, struct)

        return data

    def save_gui_config(self):
        """
        Save the GUI configuration
        :return:
        """
        data = self.get_gui_config_data()
        with open(self.config_file_path(), "w") as f:
            f.write(json.dumps(data, indent=4))

    def apply_gui_config(self, data: Dict[str, Dict[str, Any]]):
        """
        Apply GUI configuration dictionary
        :param data: GUI configuration dictionary
        """

        struct = self.get_config_structure()
        config_data_to_struct(data_=data, struct_=struct)

        # light / dark mode
        if self.ui.dark_mode_checkBox.isChecked():
            set_dark_mode()
        else:
            set_light_mode()

    def load_gui_config(self) -> None:
        """
        Load GUI configuration from the local user folder
        """
        if self.config_file_exists():
            with open(self.config_file_path(), "r") as f:
                try:
                    data = json.load(f)
                    self.apply_gui_config(data=data)
                    self.change_theme_mode()
                except json.decoder.JSONDecodeError as e:
                    print(e)
                    self.save_gui_config()
                    print("GUI config file was erroneous, wrote a new one")

    def snapshot_datetime_changed(self):
        """
        Upon change of the snapshot datetime, change the circuit snapshot datetime
        """
        date_time_value = self.ui.snapshot_dateTimeEdit.dateTime().toPython()

        self.circuit.snapshot_time = date_time_value

    def add_plugins(self):
        """
        Add the plugins information and create the menu entries
        """
        self.ui.menuplugins.clear()

        add_menu_entry(menu=self.ui.menuplugins,
                       text="Reload",
                       icon_path=":/Icons/icons/undo.png",
                       function_ptr=self.add_plugins)

        self.plugins_info.read()  # force refresh

        self.plugins_investment_evaluation_method_dict = dict()

        # for every plugin...
        for key, plugin_info in self.plugins_info.plugins.items():

            # maybe add the main function
            if plugin_info.main_fcn.function_ptr is not None:
                """
                Really hard core magic to avoid lambdas shadow each other due to late binding
                
                lambda e, func=func: func(self)
                
                explanation:
                - e is a bool parameter that the QAction sends when triggered
                - func=func is there for the lambda to force the usage of the value of func 
                  during the iteration and not after the loop
                - func(self) is then what I wanted to lambda in the first place                
                """
                add_menu_entry(
                    menu=self.ui.menuplugins,
                    text=plugin_info.name,
                    icon_path=":/Icons/icons/plugin.png",
                    icon_pixmap=plugin_info.icon,
                    function_ptr=lambda: self.launch_plugin(plugin_info.main_fcn)
                )

            # maybe add the investments function
            if plugin_info.investments_fcn.function_ptr is not None:
                self.plugins_investment_evaluation_method_dict[
                    plugin_info.investments_fcn.alias
                ] = plugin_info.investments_fcn.function_ptr

    def launch_plugin(self, fcn: PluginFunction):
        """
        Action wrapper to launch the plugin
        :param fcn: some PluginFunction
        """

        # call the main function of the plugin
        ret = fcn.get_pointer_lambda(gui_instance=self)()

        if fcn.call_gui and ret is not None:
            if hasattr(ret, "show"):
                self.plugin_windows_list.append(ret)  # This avoids the window to be garbage collected and be displayed
                ret.show()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.  
# SPDX-License-Identifier: MPL-2.0

from __future__ import annotations
from typing import TYPE_CHECKING

from PySide6 import QtWidgets
from PySide6.QtCore import QPointF
from PySide6.QtGui import QPolygonF
from VeraGrid.Gui.gui_functions import add_menu_entry
from VeraGrid.Gui.Diagrams.generic_graphics import Polygon
from VeraGrid.Gui.Diagrams.MapWidget.Injections.map_injections_template_graphics import MapInjectionTemplateGraphicItem
from VeraGrid.Gui.RmsModelEditor.dynamic_block_editor import DynamicBlockEditorGUI
from VeraGridEngine.Devices.Injections.load import Load
from VeraGridEngine.enumerations import DeviceType

if TYPE_CHECKING:  # Only imports the below statements during type checking
    from VeraGrid.Gui.Diagrams.MapWidget.grid_map_widget import GridMapWidget


class MapLoadGraphicItem(MapInjectionTemplateGraphicItem):

    def __init__(self, api_object: Load,
                 editor: GridMapWidget,
                 lat: float,
                 lon: float,
                 size: float = 0.8,
                 draw_labels: bool = True):
        """

        :param api_object:
        :param editor:
        """
        MapInjectionTemplateGraphicItem.__init__(self,
                                                 api_object=api_object,
                                                 editor=editor,
                                                 lat=lat,
                                                 lon=lon,
                                                 size=size,
                                                 draw_labels=draw_labels)

    @property
    def api_object(self) -> Load:
        return self._api_object

    def contextMenuEvent(self, event: QtWidgets.QGraphicsSceneContextMenuEvent):
        """
        Display context menu
        @param event:
        @return:
        """
        if self.api_object is not None:
            menu = self.get_base_context_menu()
            menu.addSection("Load")

            add_menu_entry(menu=menu,
                           text="Rms Editor",
                           function_ptr=self.edit_rms,
                           icon_path=":/Icons/icons/edit.png")

            menu.exec(event.screenPos())
        else:
            self.editor.gui.show_error_toast("The graphic has no API object!")

    def edit_rms(self):
        """

        :return:
        """
        # load templates
        templates = self.editor.circuit.rms_models

        # select line templates
        templ_catalogue = dict()
        templ_list = []
        for templ in templates:
            if templ.tpe == DeviceType.LoadDevice:
                templ_list.append(templ.name)
                templ_catalogue[templ.name] = templ

        # prompt RmsModelEditorGUI
        rms_model_editor = DynamicBlockEditorGUI(
            var_factory=self.editor.circuit.rms_var_factory,
            block=self.api_object.rms_model,
            templates_list=templ_list,
            api_object=self.api_object,
            main_editor=True,
        )
        rms_model_editor.show()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'coordinates_input_gui.ui'
##
## Created by: Qt User Interface Compiler version 6.9.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QDialog,
    QFrame, QGridLayout, QHeaderView, QLabel,
    QPushButton, QRadioButton, QSizePolicy, QSpacerItem,
    QSplitter, QTableView, QVBoxLayout, QWidget)
from VeraGrid.Gui.Icons.icons_rc import *

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(853, 420)
        self.verticalLayout = QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(-1, 0, -1, 9)
        self.splitter = QSplitter(Dialog)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.frame_4 = QFrame(self.splitter)
        self.frame_4.setObjectName(u"frame_4")
        self.frame_4.setFrameShape(QFrame.NoFrame)
        self.frame_4.setFrameShadow(QFrame.Raised)
        self.verticalLayout_2 = QVBoxLayout(self.frame_4)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.frame_2 = QFrame(self.frame_4)
        self.frame_2.setObjectName(u"frame_2")
        self.frame_2.setFrameShape(QFrame.NoFrame)
        self.frame_2.setFrameShadow(QFrame.Raised)
        self.gridLayout_2 = QGridLayout(self.frame_2)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setVerticalSpacing(2)
        self.gridLayout_2.setContentsMargins(-1, 0, -1, 0)
        self.nameComboBox = QComboBox(self.frame_2)
        self.nameComboBox.setObjectName(u"nameComboBox")

        self.gridLayout_2.addWidget(self.nameComboBox, 5, 1, 1, 2)

        self.longitudeCheckBox = QCheckBox(self.frame_2)
        self.longitudeCheckBox.setObjectName(u"longitudeCheckBox")

        self.gridLayout_2.addWidget(self.longitudeCheckBox, 15, 0, 2, 1)

        self.latitudeComboBox = QComboBox(self.frame_2)
        self.latitudeComboBox.setObjectName(u"latitudeComboBox")

        self.gridLayout_2.addWidget(self.latitudeComboBox, 14, 1, 1, 2)

        self.open_button = QPushButton(self.frame_2)
        self.open_button.setObjectName(u"open_button")
        icon = QIcon()
        icon.addFile(u":/Icons/icons/import_profiles.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.open_button.setIcon(icon)

        self.gridLayout_2.addWidget(self.open_button, 2, 0, 1, 1)

        self.longitudeComboBox = QComboBox(self.frame_2)
        self.longitudeComboBox.setObjectName(u"longitudeComboBox")

        self.gridLayout_2.addWidget(self.longitudeComboBox, 16, 1, 1, 2)

        self.nameRadioButton = QRadioButton(self.frame_2)
        self.nameRadioButton.setObjectName(u"nameRadioButton")
        self.nameRadioButton.setChecked(True)

        self.gridLayout_2.addWidget(self.nameRadioButton, 5, 0, 1, 1)

        self.label_4 = QLabel(self.frame_2)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_2.addWidget(self.label_4, 7, 0, 1, 1)

        self.codeComboBox = QComboBox(self.frame_2)
        self.codeComboBox.setObjectName(u"codeComboBox")

        self.gridLayout_2.addWidget(self.codeComboBox, 6, 1, 1, 2)

        self.yCheckBox = QCheckBox(self.frame_2)
        self.yCheckBox.setObjectName(u"yCheckBox")

        self.gridLayout_2.addWidget(self.yCheckBox, 11, 0, 2, 1)

        self.xComboBox = QComboBox(self.frame_2)
        self.xComboBox.setObjectName(u"xComboBox")

        self.gridLayout_2.addWidget(self.xComboBox, 10, 1, 1, 2)

        self.label_2 = QLabel(self.frame_2)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_2.addWidget(self.label_2, 3, 0, 1, 1)

        self.yComboBox = QComboBox(self.frame_2)
        self.yComboBox.setObjectName(u"yComboBox")

        self.gridLayout_2.addWidget(self.yComboBox, 12, 1, 1, 2)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.gridLayout_2.addItem(self.verticalSpacer, 17, 0, 1, 3)

        self.codeRadioButton = QRadioButton(self.frame_2)
        self.codeRadioButton.setObjectName(u"codeRadioButton")

        self.gridLayout_2.addWidget(self.codeRadioButton, 6, 0, 1, 1)

        self.latitudeCheckBox = QCheckBox(self.frame_2)
        self.latitudeCheckBox.setObjectName(u"latitudeCheckBox")

        self.gridLayout_2.addWidget(self.latitudeCheckBox, 13, 0, 2, 1)

        self.xCheckBox = QCheckBox(self.frame_2)
        self.xCheckBox.setObjectName(u"xCheckBox")

        self.gridLayout_2.addWidget(self.xCheckBox, 9, 0, 2, 1)

        self.label_3 = QLabel(self.frame_2)
        self.label_3.setObjectName(u"label_3")

        self.gridLayout_2.addWidget(self.label_3, 4, 0, 1, 3)

        self.label = QLabel(self.frame_2)
        self.label.setObjectName(u"label")

        self.gridLayout_2.addWidget(self.label, 1, 0, 1, 1)

        self.label_5 = QLabel(self.frame_2)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_2.addWidget(self.label_5, 8, 0, 1, 3)


        self.verticalLayout_2.addWidget(self.frame_2)

        self.splitter.addWidget(self.frame_4)
        self.frame_6 = QFrame(self.splitter)
        self.frame_6.setObjectName(u"frame_6")
        self.frame_6.setFrameShape(QFrame.NoFrame)
        self.frame_6.setFrameShadow(QFrame.Raised)
        self.verticalLayout_3 = QVBoxLayout(self.frame_6)
        self.verticalLayout_3.setSpacing(2)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.assignation_table = QTableView(self.frame_6)
        self.assignation_table.setObjectName(u"assignation_table")

        self.verticalLayout_3.addWidget(self.assignation_table)

        self.frame_9 = QFrame(self.frame_6)
        self.frame_9.setObjectName(u"frame_9")
        self.frame_9.setFrameShape(QFrame.NoFrame)
        self.frame_9.setFrameShadow(QFrame.Raised)
        self.gridLayout = QGridLayout(self.frame_9)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout.addItem(self.horizontalSpacer_3, 0, 1, 1, 1)

        self.acceptButton = QPushButton(self.frame_9)
        self.acceptButton.setObjectName(u"acceptButton")
        icon1 = QIcon()
        icon1.addFile(u":/Icons/icons/gear.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.acceptButton.setIcon(icon1)

        self.gridLayout.addWidget(self.acceptButton, 0, 2, 1, 1)

        self.refreshButton = QPushButton(self.frame_9)
        self.refreshButton.setObjectName(u"refreshButton")
        icon2 = QIcon()
        icon2.addFile(u":/Icons/icons/transform.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.refreshButton.setIcon(icon2)

        self.gridLayout.addWidget(self.refreshButton, 0, 0, 1, 1)


        self.verticalLayout_3.addWidget(self.frame_9)

        self.splitter.addWidget(self.frame_6)

        self.verticalLayout.addWidget(self.splitter)


        self.retranslateUi(Dialog)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Dialog", None))
        self.longitudeCheckBox.setText(QCoreApplication.translate("Dialog", u"Bus longitude", None))
        self.open_button.setText(QCoreApplication.translate("Dialog", u"Load file", None))
        self.nameRadioButton.setText(QCoreApplication.translate("Dialog", u"Name", None))
        self.label_4.setText("")
        self.yCheckBox.setText(QCoreApplication.translate("Dialog", u"Bus y position", None))
        self.label_2.setText("")
        self.codeRadioButton.setText(QCoreApplication.translate("Dialog", u"Code", None))
        self.latitudeCheckBox.setText(QCoreApplication.translate("Dialog", u"Bus latitude", None))
        self.xCheckBox.setText(QCoreApplication.translate("Dialog", u"Bus x position", None))
        self.label_3.setText(QCoreApplication.translate("Dialog", u"Match mathod:", None))
        self.label.setText("")
        self.label_5.setText(QCoreApplication.translate("Dialog", u"Assigning magnitudes", None))
#if QT_CONFIG(tooltip)
        self.acceptButton.setToolTip(QCoreApplication.translate("Dialog", u"Do it!", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        self.acceptButton.setStatusTip("")
#endif // QT_CONFIG(statustip)
        self.acceptButton.setText(QCoreApplication.translate("Dialog", u"Accept", None))
        self.refreshButton.setText(QCoreApplication.translate("Dialog", u"Match", None))
    # retranslateUi




# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations
from typing import Tuple, List, TYPE_CHECKING

from VeraGridEngine.enumerations import DeviceType
from PySide6.QtWidgets import QMenu, QGraphicsSceneContextMenuEvent
from VeraGrid.Gui.gui_functions import add_menu_entry
from PySide6 import QtWidgets
from PySide6.QtCore import Qt, QPointF
from PySide6.QtGui import QBrush, QColor

from VeraGrid.Gui.messages import yes_no_question
from VeraGridEngine.Devices.Branches.line_locations import LineLocation
from VeraGrid.Gui.Diagrams.MapWidget.Branches.map_line_container import MapLineContainer
from VeraGrid.Gui.Diagrams.MapWidget.Substation.node_template import NodeTemplate

if TYPE_CHECKING:  # Only imports the below statements during type checking
    from VeraGrid.Gui.Diagrams.MapWidget.grid_map_widget import GridMapWidget


class LineLocationGraphicItem(QtWidgets.QGraphicsEllipseItem, NodeTemplate):
    """
      Represents a block in the diagram
      Has an x and y and width and height
      width and height can only be adjusted with a tip in the lower right corner.

      - in and output ports
      - parameters
      - description
    """

    def __init__(self,
                 editor: GridMapWidget,
                 line_container: MapLineContainer,
                 api_object: LineLocation,
                 lat: float,
                 lon: float,
                 index: int,
                 r: float = 0.006,
                 draw_labels: bool = True):
        """

        :param editor:
        :param line_container:
        :param api_object:
        :param lat:
        :param lon:
        :param r:
        :param draw_labels:
        """
        QtWidgets.QGraphicsEllipseItem.__init__(self)
        NodeTemplate.__init__(self,
                              api_object=api_object,
                              editor=editor,
                              draw_labels=draw_labels,
                              lat=lat,
                              lon=lon)

        self.lat = lat
        self.lon = lon
        self.x, self.y = editor.to_x_y(lat=lat, lon=lon)
        self.radius = r
        self.draw_labels = True

        self.line_container: MapLineContainer = line_container

        self.index = index

        self.resize(r)
        self.setAcceptHoverEvents(True)  # Enable hover events for the item
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsMovable)  # Allow moving the node
        self.setFlag(QtWidgets.QGraphicsItem.GraphicsItemFlag.ItemIsSelectable)  # Allow selecting the node

        # self.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
        self.setCursor(Qt.CursorShape.PointingHandCursor)

        self.hovered = False
        self.enabled = True

        # Create a pen with reduced line width
        self.change_pen_width(0)
        self.colorInner = QColor(100, 100, 100, 100)
        self.colorBorder = QColor(100, 100, 100, 100)
        self.setZValue(1)

        # Assign color to the node
        self.set_default_color()

    @property
    def api_object(self) -> LineLocation:
        return self._api_object

    @property
    def editor(self) -> GridMapWidget:
        return self._editor

    def get_associated_widgets(self) -> List[MapLineContainer]:
        """
        This forwards to the map line container for the appropriate deletion of everything
        :return:
        """
        return [self.line_container]

    def get_center_pos(self) -> QPointF:
        """

        :return:
        """
        x = self.rect().x() + self.rect().width() / 2
        y = self.rect().y() + self.rect().height() / 2
        return QPointF(x, y)

    def update_real_pos(self) -> None:
        """

        :return:
        """
        real_position = self.pos()
        center_point = self.get_pos()
        self.x = center_point.x() + real_position.x()
        self.y = center_point.y() + real_position.y()

    def update_position(self) -> None:
        """

        :return:
        """

        if self.enabled:
            self.update_real_pos()
            self.line_container.update_connectors()

    def update_position_at_the_diagram(self) -> None:
        """
        This function updates the position of this graphical element in the diagram
        """
        real_position = self.pos()
        center_point = self.get_pos()

        self.lat, self.lon = self.editor.to_lat_lon(x=center_point.x() + real_position.x(),
                                                     y=center_point.y() + real_position.y())

        # print(f'Updating node position id:{self.api_object.idtag}, lat:{self.lat}, lon:{self.lon}')

        self.editor.update_diagram_element(device=self.api_object,
                                            latitude=self.lat,
                                            longitude=self.lon,
                                            graphic_object=self)

    def update_database_position(self) -> None:
        """
        This function updates the position of this graphical element in the database
        """
        real_position = self.pos()
        center_point = self.get_pos()

        self.lat, self.lon = self.editor.to_lat_lon(x=center_point.x() + real_position.x(),
                                                     y=center_point.y() + real_position.y())

        # print(f'Updating node position id:{self.api_object.idtag}, lat:{self.lat}, lon:{self.lon}')

        self.api_object.lat = self.lat
        self.api_object.long = self.lon

    def move_to_api_coordinates(self, question: bool = True):
        """
        Function to move the graphics to the Database location
        :return:
        """
        if question:
            ok = yes_no_question(f"Move substation {self.api_object.name} graphics to it's database coordinates?",
                                 "Move substation graphics")

            if ok:
                x, y = self.move_to(lat=self.api_object.lat,
                                    lon=self.api_object.long)  # this moves the vl too
                self.set_callbacks(x, y)
        else:
            x, y = self.move_to(lat=self.api_object.lat, lon=self.api_object.long)  # this moves the vl too
            self.set_callbacks(x, y)

    def move_to(self, lat: float, lon: float) -> Tuple[float, float]:
        """

        :param lat:
        :param lon:
        :return: x, y
        """
        x, y = self.editor.to_x_y(lat=lat, lon=lon)  # upper left corner

        self.setRect(
            x - self.rect().width() / 2,
            y - self.rect().height() / 2,
            self.rect().width(),
            self.rect().height()
        )

        return x, y

    def mouseMoveEvent(self, event: QtWidgets.QGraphicsSceneMouseEvent) -> None:
        """
        Event handler for mouse move events.
        """
        if self.enabled:
            pos = self.mapToParent(event.pos())
            x = pos.x() - self.rect().width() / 2
            y = pos.y() - self.rect().height() / 2
            self.setRect(x, y, self.rect().width(), self.rect().height())
            self.set_callbacks(pos.x(), pos.y())

            if self.hovered and self.enabled:
                self.update_position()

    def mousePressEvent(self, event: QtWidgets.QGraphicsSceneMouseEvent) -> None:
        """
        Event handler for mouse press events.
        """
        super().mousePressEvent(event)
        if self.enabled:
            self.editor.map.view.disable_move = True
            if event.button() == Qt.MouseButton.RightButton:
                pass

    def mouseReleaseEvent(self, event: QtWidgets.QGraphicsSceneMouseEvent) -> None:
        """
        Event handler for mouse release events.
        """
        super().mouseReleaseEvent(event)
        self.editor.disableMove = True
        self.update_position_at_the_diagram()
        # self.update_database_position()

    def hoverEnterEvent(self, event: QtWidgets.QGraphicsSceneHoverEvent) -> None:
        """
        Event handler for when the mouse enters the item.
        """
        self.hovered = True
        self.set_node_color(QColor(Qt.GlobalColor.red), QColor(Qt.GlobalColor.red))

    def hoverLeaveEvent(self, event: QtWidgets.QGraphicsSceneHoverEvent) -> None:
        """
        Event handler for when the mouse leaves the item.
        """
        self.hovered = False
        self.set_default_color()

    def contextMenuEvent(self, event: QGraphicsSceneContextMenuEvent):
        """
        Event handler for context menu events.
        :param event:
        :return:
        """
        menu = QMenu()

        add_menu_entry(menu=menu,
                       text="Delete",
                       icon_path=":/Icons/icons/delete_schematic.png",
                       function_ptr=self.remove)

        add_menu_entry(menu=menu,
                       text="Transform waypoint into substation",
                       function_ptr=self.editor.transform_waypoint_to_substation,
                       icon_path=":/Icons/icons/divide.png")

        menu.addSeparator()

        has_substation = False

        for graphic_obj in self.editor._get_selected():
            if hasattr(graphic_obj, 'api_object'):
                if hasattr(graphic_obj.api_object, 'device_type'):
                    if graphic_obj.api_object.device_type == DeviceType.SubstationDevice:
                        has_substation = True

        if has_substation:

            add_menu_entry(menu=menu,
                           text="Connect line to selected substation (T-joint) at this waypoint",
                           function_ptr=self.editor.create_t_joint_to_substation,
                           icon_path=":/Icons/icons/divide.png")

        menu.exec_(event.screenPos())

    def remove(self):
        """
        Remove
        """
        self.line_container.remove_node(node=self)

    def set_node_color(self, inner_color: QColor, border_color: QColor = None) -> None:
        """

        :param inner_color:
        :param border_color:
        :return:
        """
        # Example: color assignment
        brush = QBrush(inner_color)
        self.setBrush(brush)

        if border_color is not None:
            pen = self.pen()
            pen.setColor(border_color)
            self.setPen(pen)

    def set_default_color(self) -> None:
        """

        :return:
        """
        # Example: color assignment
        self.set_node_color(self.colorInner, self.colorBorder)

    def get_pos(self) -> QPointF:
        """

        :return:
        """
        # Get the bounding rectangle of the ellipse item
        bounding_rect = self.boundingRect()

        # Calculate the center point of the bounding rectangle
        center_point = bounding_rect.center()

        return center_point

    def getRealPos(self) -> Tuple[float, float]:
        """

        :return:
        """
        self.update_real_pos()
        return self.x, self.y

    def resize(self, new_radius: float):
        """
        Resize the node.
        :param new_radius: New radius for the node.
        """
        self.radius = new_radius
        self.setRect(self.x - new_radius, self.y - new_radius, new_radius * 2, new_radius * 2)

    def change_pen_width(self, width: int):
        """
        Change the pen width for the node.
        :param width: New pen width.
        """
        pen = self.pen()
        pen.setWidth(width)
        self.setPen(pen)





# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations
from typing import TYPE_CHECKING
from PySide6 import QtWidgets
from VeraGrid.Gui.PsseDialogue.psse_import_gui import Ui_PsseImportDialog
from VeraGrid.Gui.general_dialogues import LogsDialogue
from VeraGridEngine.IO.file_open import FileOpenOptions
from VeraGridEngine.basic_structures import Logger

if TYPE_CHECKING:
    from VeraGrid.Gui.Main.SubClasses.io import IoMain


class PsseImportDialogue(QtWidgets.QDialog):
    """
    GridMergeDialogue
    """

    def __init__(self, app: IoMain, options: FileOpenOptions):
        """
        GridMergeDialogue
        :param app: App pointer
        """
        QtWidgets.QDialog.__init__(self)
        self.ui = Ui_PsseImportDialog()
        self.ui.setupUi(self)
        self.setWindowTitle('PSS/e import')
        self.setModal(True)

        self.app = app

        self.logger = Logger()
        self.logs_dialogue: LogsDialogue | None = None

        self.options = options

        self.ui.importButton.clicked.connect(self.do_it)

    def do_it(self):
        """

        :return:
        """
        self.options.psse_adjust_taps_to_discrete_positions = self.ui.adjust_taps_to_discrete_positions_checkBox.isChecked()
        self.options.psse_use_short_names = self.ui.use_short_names_checkBox.isChecked()
        self.options.psse_flatten_virtual_taps = self.ui.flatten_virtual_taps_checkBox.isChecked()
        self.close()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations

from typing import Dict, List, Union

from PySide6 import QtCore, QtWidgets

from VeraGrid.Gui.gui_functions import (FloatDelegate)
from VeraGrid.Gui.wrappable_table_model import WrappableTableModel
from VeraGridEngine.Devices.Associations.association import Association, Associations
from VeraGridEngine.Devices.Parents.editable_device import GCProp
from VeraGridEngine.Devices.types import ASSOCIATION_TYPES, ALL_DEV_TYPES


def try_convert_to_float(value: str) -> Union[float, str]:
    """

    :param value:
    :return:
    """
    try:
        return float(value)
    except ValueError:
        return value


class AssociationsModel(WrappableTableModel):
    """
    Class to populate a Qt table view with the properties of objects
    """

    def __init__(self,
                 objects: List[ALL_DEV_TYPES],
                 associated_objects: List[ASSOCIATION_TYPES],
                 gc_prop: GCProp,
                 table_view: QtWidgets.QTableView,
                 decimals: int = 2):
        """

        :param objects:
        :param associated_objects:
        :param gc_prop:
        :param table_view:
        """
        WrappableTableModel.__init__(self)

        self._table_view = table_view

        self._objects: List[ALL_DEV_TYPES] = objects

        self._associated_objects: List[ASSOCIATION_TYPES] = associated_objects

        self._associated_obj_2_idx: Dict[ASSOCIATION_TYPES, int] = dict()

        for i, obj in enumerate(self._associated_objects):
            self._associated_obj_2_idx[obj] = i

        self._gc_prop = gc_prop

        # relate the objects to the associations
        # [object index][associated object index] -> association
        self._sp_data: List[Dict[int, Association]] = list()

        for i, obj in enumerate(self._objects):
            associations_list: List[Association] = getattr(obj, self._gc_prop.name)
            entries_dict = {self._associated_obj_2_idx[asoc.api_object]: asoc for asoc in associations_list}
            self._sp_data.append(entries_dict)

        self._formatter = lambda x: f"%.2f" % x

        self._decimals = decimals

    def get_association(self, i: int, j: int) -> Union[None, Association]:
        """
        Get the association at some coordinates
        :param i: row index
        :param j: column index
        :return: Association or None
        """
        # return self._sp_data[i].get(j, None)
        associations: Associations = getattr(self._objects[i], self._gc_prop.name)
        associated_obj = self._associated_objects[j]
        return associations.at_key(associated_obj.idtag)

    def create_association(self, i: int, j: int, value: float) -> Association:
        """
        Get the association at some coordinates
        :param i: row index
        :param j: column index
        :param value: value to associate
        :return: Association
        """
        associations: Associations = getattr(self._objects[i], self._gc_prop.name)
        assoc = associations.add_object(api_object=self._associated_objects[j], val=value)
        return assoc

    def remove_association(self, i: int, j: int) -> None:
        """
        Remove the association at some coordinates
        :param i: row index
        :param j: column index
        :return: Association or None
        """
        associations: Associations = getattr(self._objects[i], self._gc_prop.name)
        associated_obj = self._associated_objects[j]
        associations.remove_by_key(associated_obj.idtag)

    def set_delegates(self) -> None:
        """
        Set the cell editor types depending on the attribute_types array
        """

        for i in range(len(self._associated_objects)):
            delegate = FloatDelegate(self._table_view, decimals=self._decimals)
            self._table_view.setItemDelegateForColumn(i, delegate)

    def update(self):
        """
        update table
        """
        # row = self.rowCount()
        # self.beginInsertRows(QtCore.QModelIndex(), row, row)
        # # whatever code
        # self.endInsertRows()

        self.layoutAboutToBeChanged.emit()
        self.layoutChanged.emit()

    def flags(self, index: QtCore.QModelIndex):
        """
        Get the display mode
        :param index:
        :return:
        """
        return QtCore.Qt.ItemFlag.ItemIsEditable | QtCore.Qt.ItemFlag.ItemIsEnabled | QtCore.Qt.ItemFlag.ItemIsSelectable

    def rowCount(self, parent: QtCore.QModelIndex = None) -> int:
        """
        Get number of rows
        :param parent:
        :return:
        """
        return len(self._objects)

    def columnCount(self, parent: QtCore.QModelIndex = None) -> int:
        """
        Get number of columns
        :param parent:
        :return:
        """
        return len(self._associated_objects)

    def data(self, index: QtCore.QModelIndex, role=None):
        """
        Get the data to display
        :param index:
        :param role:
        :return:
        """
        if index.isValid():
            if role == QtCore.Qt.ItemDataRole.DisplayRole:

                association = self.get_association(i=index.row(), j=index.column())
                if association is None:
                    return "-"
                else:
                    return str(association.value)

        return None

    def setData(self, index: QtCore.QModelIndex, value: Union[float, str], role: Union[int, None] = None) -> bool:
        """
        Set data by simple editor (whatever text)
        :param index:
        :param value:
        :param role:
        :return:
        """
        if len(self._objects) == 0 or len(self._associated_objects) == 0:
            return True

        value2 = try_convert_to_float(value)

        if isinstance(value2, (float, int)):
            association = self.get_association(i=index.row(), j=index.column())
            if association is None:
                # create an association
                self.create_association(i=index.row(), j=index.column(), value=float(value2))
                print(f"Created association {self._objects[index.row()]} "
                      f"with {self._associated_objects[index.column()]}")
            else:
                # there was a previous association, change the value
                association.value = value2

        elif isinstance(value2, str):
            if str(value2).strip() == "-":
                self.remove_association(i=index.row(), j=index.column())
                print(f"Removed association {self._objects[index.row()]} "
                      f"with {self._associated_objects[index.column()]}")

        return True

    def headerData(self,
                   section: int,
                   orientation: QtCore.Qt.Orientation,
                   role=QtCore.Qt.ItemDataRole.DisplayRole):
        """
        Get the headers to display
        :param section:
        :param orientation:
        :param role:
        :return:
        """
        if self._hide_headers_mode is True:
            return None

        if role == QtCore.Qt.ItemDataRole.DisplayRole:

            # Normal
            if orientation == QtCore.Qt.Orientation.Horizontal:
                # return f"H{section}"  #
                # if 0 <= section < len(self._associated_objects):
                return self._associated_objects[section].name

            elif orientation == QtCore.Qt.Orientation.Vertical:
                # return f"V{section}"  #
                # if 0 <= section < len(self._objects):
                return self._objects[section].name

        # add a tooltip
        # elif role == QtCore.Qt.ItemDataRole.ToolTipRole:
        #     if orientation == QtCore.Qt.Orientation.Horizontal:
        #         return f"{section}:{self.associated_objects[section].name}"
        #
        #     elif orientation == QtCore.Qt.Orientation.Vertical:
        #         return f"{section}:{self.objects[section].name}"

        return None



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.  
# SPDX-License-Identifier: MPL-2.0

import os
from typing import Union, List, Dict
from PySide6.QtCore import QThread, Signal

from VeraGrid.Session.session import SimulationSession
from VeraGridEngine.basic_structures import Logger
from VeraGridEngine.IO.veragrid.zip_interface import get_session_tree, load_session_driver_objects
from VeraGridEngine.IO.file_open import FileOpen, FileOpenOptions
from VeraGridEngine.IO.file_save import FileSavingOptions, FileSave
from VeraGridEngine.Devices.multi_circuit import MultiCircuit
from VeraGridEngine.IO.cim.cgmes.cgmes_circuit import CgmesCircuit
from VeraGridEngine.data_logger import DataLogger


class FileOpenThread(QThread):
    """
    FileOpenThread
    """
    progress_signal = Signal(float)
    progress_text = Signal(str)
    done_signal = Signal()

    def __init__(self,
                 file_name: Union[str, List[str]],
                 previous_circuit: Union[MultiCircuit, None] = None,
                 options: FileOpenOptions = None):
        """
        Constructor
        :param file_name: file name (s)
        :param previous_circuit: we could provide an additional circuit.
                                This is relevant if loading grid incrementals that rely on existing data
        """
        QThread.__init__(self)

        self.file_name: Union[str, List[str]] = file_name

        self.valid = False

        self.logger = Logger()

        self.circuit: Union[MultiCircuit, None] = None

        self.options = options if options is not None else FileOpenOptions()

        self.cgmes_circuit: Union[CgmesCircuit, None] = None

        self.cgmes_logger: DataLogger = DataLogger()

        self.json_files = dict()

        self._previous_circuit = previous_circuit

        self.__cancel__ = False

    def get_session_tree(self) -> Dict[str, Union[SimulationSession]]:
        """
        Get the session tree structure from a VeraGrid file
        :return:
        """
        if isinstance(self.file_name, str):
            if self.file_name.endswith('.gridcal') or self.file_name.endswith('.veragrid'):
                return get_session_tree(self.file_name)
            else:
                return dict()
        else:
            return dict()

    def load_session_objects(self, session_name: str, study_name: str):
        """
        Load the numpy objects of the session
        :param session_name: Name of the session (i.e. GUI Session)
        :param study_name: Name of the study i.e Power Flow)
        :return: Dictionary (name: array)
        """
        if isinstance(self.file_name, str):
            if self.file_name.endswith('.gridcal') or self.file_name.endswith('.veragrid'):
                return load_session_driver_objects(file_name_zip=self.file_name,
                                                   session_name=session_name,
                                                   study_name=study_name)
            else:
                return dict()
        else:
            return dict()

    def run(self) -> None:
        """
        run the file open procedure
        """
        self.circuit = MultiCircuit()

        if isinstance(self.file_name, list):
            path, fname = os.path.split(self.file_name[0])
            self.progress_text.emit('Loading ' + fname + '...')
        else:
            path, fname = os.path.split(self.file_name)
            self.progress_text.emit('Loading ' + fname + '...')

        self.logger = Logger()

        file_handler = FileOpen(file_name=self.file_name,
                                previous_circuit=self._previous_circuit,
                                options=self.options)

        if self.options.crash_on_errors:
            self.circuit = file_handler.open(text_func=self.progress_text.emit,
                                             progress_func=self.progress_signal.emit)
        else:
            try:
                self.circuit = file_handler.open(text_func=self.progress_text.emit,
                                                 progress_func=self.progress_signal.emit)
            except ValueError as e:
                self.valid = False
                self.logger.add_error(msg=str(e))
                self.progress_text.emit('Error loading')
                self.done_signal.emit()
                return

        self.json_files = file_handler.json_files

        self.cgmes_circuit = file_handler.cgmes_circuit
        self.cgmes_logger = file_handler.cgmes_logger

        self.logger += file_handler.logger
        self.valid = True

        # post events
        self.progress_text.emit('Done!')

        self.done_signal.emit()

    def cancel(self) -> None:
        """
        Set the cancel flag
        """
        self.__cancel__ = True


class FileSaveThread(QThread):
    """
    Thread to save
    """
    progress_signal = Signal(float)
    progress_text = Signal(str)
    done_signal = Signal()

    def __init__(self,
                 circuit: MultiCircuit,
                 file_name: str,
                 options: FileSavingOptions):
        """
        Constructor
        :param circuit: MultiCircuit instance
        :param file_name: name of the file where to save
        :param options: FileSavingOptions
        """
        QThread.__init__(self)

        self.circuit = circuit

        self.file_name = file_name

        self.valid = False

        self.options = options

        self.logger = Logger()

        self.error_msg = ''

        self.__cancel__ = False

    def get_session_tree(self) -> Dict:
        """
        Get the session tree structure from a VeraGrid file
        :return:
        """
        if isinstance(self.file_name, str):
            if self.file_name.endswith('.gridcal') or self.file_name.endswith('.veragrid'):
                return get_session_tree(self.file_name)
            else:
                return dict()
        else:
            return dict()

    def load_session_objects(self, session_name: str, study_name: str):
        """
        Load the numpy objects of the session
        :param session_name: Name of the session (i.e. GUI Session)
        :param study_name: Name of the study i.e Power Flow)
        :return: Dictionary (name: array)
        """
        if isinstance(self.file_name, str):
            if self.file_name.endswith('.gridcal') or self.file_name.endswith('.veragrid'):
                return load_session_driver_objects(self.file_name, session_name, study_name)
            else:
                return dict()
        else:
            return dict()

    def run(self) -> None:
        """
        run the file save procedure
        @return:
        """

        path, fname = os.path.split(self.file_name)

        self.progress_text.emit('Flushing ' + fname + ' into ' + fname + '...')

        self.logger = Logger()

        file_handler = FileSave(circuit=self.circuit,
                                file_name=self.file_name,
                                options=self.options,
                                text_func=self.progress_text.emit,
                                progress_func=self.progress_signal.emit)
        try:
            self.logger = file_handler.save()
        except PermissionError:
            self.logger.add_error("File permission denied. Do you have the file open? Do you have write permissions?")

        self.valid = True

        # post events
        self.progress_text.emit('Done!')

        self.done_signal.emit()

    def cancel(self):
        """
        Activate the cancel flag
        """
        self.__cancel__ = True



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
# SPDX-License-Identifier: MPL-2.0
from __future__ import annotations

import io
import numpy as np
import pandas as pd
from typing import List, Union, Any, Dict
from datetime import datetime
from PySide6 import QtCore, QtGui, QtWidgets
from PySide6.QtWidgets import (QApplication, QDialog, QTableView, QVBoxLayout, QPushButton, QHBoxLayout,
                               QLabel, QComboBox, QSpacerItem, QSizePolicy)

from PySide6.QtCore import Qt, QAbstractTableModel, QModelIndex
from VeraGridEngine.basic_structures import Logger, IntVec
from VeraGridEngine.Devices.types import ALL_DEV_TYPES
from VeraGrid.Gui.gui_functions import get_list_model, get_checked_indices, get_chck_list_model
from VeraGrid.Gui.object_model import ObjectsModel
from VeraGridEngine.enumerations import FaultType, MethodShortCircuit, PhasesShortCircuit, FileType, CGMESVersions


class CenteredDialog(QDialog):
    """
    Class to make the dialogues centered
    """

    def __init__(self, parent=None):
        super().__init__(parent)

    def showEvent(self, event):
        """

        :param event:
        :return:
        """
        super().showEvent(event)
        if self.parent():
            parent_geo = self.parent().geometry()
            self.move(
                parent_geo.center().x() - self.width() // 2,
                parent_geo.center().y() - self.height() // 2
            )


class NewProfilesStructureDialogue(CenteredDialog):
    """
    New profile dialogue window
    """

    def __init__(self) -> None:
        super(NewProfilesStructureDialogue, self).__init__()
        self.setObjectName("self")
        # self.resize(200, 71)
        # self.setMinimumSize(QtCore.QSize(200, 71))
        # self.setMaximumSize(QtCore.QSize(200, 71))
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        # icon = QtGui.QIcon()
        # icon.addPixmap(QtGui.QPixmap("Icons/Plus-32.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        # self.setWindowIcon(icon)
        self.layout = QtWidgets.QVBoxLayout(self)

        # calendar
        self.calendar = QtWidgets.QDateTimeEdit()
        d = datetime.today()
        self.calendar.setDateTime(QtCore.QDateTime(d.year, 1, 1, 00, 00, 00))

        # number of time steps
        self.steps_spinner = QtWidgets.QSpinBox()
        self.steps_spinner.setMinimum(1)
        self.steps_spinner.setMaximum(9999999)
        self.steps_spinner.setValue(1)

        # time step length
        self.step_length = QtWidgets.QDoubleSpinBox()
        self.step_length.setMinimum(1)
        self.step_length.setMaximum(60)
        self.step_length.setValue(1)

        # units combo box
        self.units = QtWidgets.QComboBox()
        self.units.setModel(get_list_model(['h', 'm', 's']))

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # labels

        # add all to the GUI
        self.layout.addWidget(QtWidgets.QLabel("Start date"))
        self.layout.addWidget(self.calendar)

        self.layout.addWidget(QtWidgets.QLabel("Number of time steps"))
        self.layout.addWidget(self.steps_spinner)

        self.layout.addWidget(QtWidgets.QLabel("Time step length"))
        self.layout.addWidget(self.step_length)

        self.layout.addWidget(QtWidgets.QLabel("Time units"))
        self.layout.addWidget(self.units)

        self.layout.addWidget(self.accept_btn)

        self.setLayout(self.layout)

        self.setWindowTitle('New profiles structure')

    def accept_click(self):
        self.accept()

    def get_values(self):
        steps = self.steps_spinner.value()

        step_length = self.step_length.value()

        step_unit = self.units.currentText()

        time_base = self.calendar.dateTime()

        return steps, step_length, step_unit, time_base.toPython()


def fill_tree_from_logs(logger: Logger):
    """
    Fill logger tree
    :param logger: Logger instance
    :return: QStandardItemModel instance
    """
    d = logger.to_dict()
    editable = False
    model = QtGui.QStandardItemModel()
    model.setHorizontalHeaderLabels(['Time', 'Class', 'Property', 'Device', 'Value', 'Expected value'])
    parent = model.invisibleRootItem()

    for severity, messages_dict in d.items():
        severity_child = QtGui.QStandardItem(severity)

        # print(severity)

        for message, data_list in messages_dict.items():
            message_child = QtGui.QStandardItem(message)

            # print('\t', message)
            try:
                for time, cls, prop, elm, value, expected_value in data_list:
                    # print('\t', '\t', time, elm, value, expected_value)

                    time_child = QtGui.QStandardItem(time)
                    time_child.setEditable(editable)

                    elm_cls = QtGui.QStandardItem(cls)
                    elm_cls.setEditable(editable)

                    elm_prop = QtGui.QStandardItem(prop)
                    elm_prop.setEditable(editable)

                    elm_child = QtGui.QStandardItem(elm)
                    elm_child.setEditable(editable)

                    value_child = QtGui.QStandardItem(value)
                    value_child.setEditable(editable)

                    expected_val_child = QtGui.QStandardItem(expected_value)
                    expected_val_child.setEditable(editable)

                    message_child.appendRow([time_child, elm_cls, elm_prop, elm_child, value_child, expected_val_child])
            except OverflowError as e:
                print(e)

            message_child.setEditable(editable)

            severity_child.appendRow(message_child)

        severity_child.setEditable(editable)
        parent.appendRow(severity_child)

    return model


class MTreeExpandHook(QtCore.QObject):
    """
    MTreeExpandHook( QTreeView )
    """

    def __init__(self, tree):
        super(MTreeExpandHook, self).__init__()
        self.setParent(tree)
        # NOTE viewport for click event listen
        tree.viewport().installEventFilter(self)
        self.tree = tree

    def eventFilter(self, receiver, event):
        if (
                # NOTE mouse left click
                event.type() == QtCore.QEvent.Type.MouseButtonPress
                # NOTE keyboard shift press
                and event.modifiers() & QtCore.Qt.KeyboardModifier.ShiftModifier
        ):
            # NOTE get mouse local position
            pos = self.tree.mapFromGlobal(QtGui.QCursor.pos())
            index = self.tree.indexAt(pos)
            if not self.tree.isExpanded(index):
                # NOTE expand all child
                self.tree.expandRecursively(index)
                return True
        return super(MTreeExpandHook, self).eventFilter(self.tree, event)


class LogsDialogue(CenteredDialog):
    """
    New profile dialogue window
    """

    def __init__(self, name: str, logger: Logger, expand_all=True):
        super(LogsDialogue, self).__init__()
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.logger: Logger = logger

        # logs_list
        self.logs_table = QtWidgets.QTreeView()
        model = fill_tree_from_logs(logger)
        self.logs_table.setModel(model)
        self.logs_table.setFirstColumnSpanned(0, QtCore.QModelIndex(), True)
        self.logs_table.setFirstColumnSpanned(1, QtCore.QModelIndex(), True)
        self.logs_table.setAnimated(True)
        # MTreeExpandHook(self.logs_table)

        if expand_all:
            self.logs_table.expandAll()

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        self.save_btn = QtWidgets.QPushButton()
        self.save_btn.setText('Save')
        self.save_btn.clicked.connect(self.save_click)

        self.copy_btn = QtWidgets.QPushButton()
        self.copy_btn.setText('Copy')
        self.copy_btn.clicked.connect(self.copy_click)

        self.btn_frame = QtWidgets.QFrame()
        self.btn_layout = QtWidgets.QHBoxLayout(self.btn_frame)
        self.btn_spacer = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Policy.Expanding)
        self.btn_layout.addWidget(self.save_btn)
        self.btn_layout.addWidget(self.copy_btn)
        self.btn_layout.addSpacerItem(self.btn_spacer)
        self.btn_layout.addWidget(self.accept_btn)
        self.btn_frame.setLayout(self.btn_layout)

        # add all to the GUI
        self.main_layout.addWidget(self.logs_table)
        self.main_layout.addWidget(self.btn_frame)

        self.setLayout(self.main_layout)

        self.setWindowTitle(name)

        h = 400
        self.resize(int(1.61 * h), h)

    def accept_click(self) -> None:
        """
        Accept and close
        """
        self.accept()

    def save_click(self):
        """
        Save the logs to excel or CSV
        """
        file, filter_ = QtWidgets.QFileDialog.getSaveFileName(self, "Export results", '',
                                                              filter="CSV (*.csv);;Excel files (*.xlsx)", )

        if file != '':
            if 'xlsx' in filter_:
                f = file
                if not f.endswith('.xlsx'):
                    f += '.xlsx'
                self.logger.to_xlsx(f)

            if 'csv' in filter_:
                f = file
                if not f.endswith('.csv'):
                    f += '.csv'
                self.logger.to_csv(f)

    def copy_click(self):
        """
        Copy logs to the clipboard
        """
        df = self.logger.to_df()
        s = io.StringIO()
        df.to_csv(s, sep='\t')
        txt = s.getvalue()

        # copy to clipboard
        cb = QtWidgets.QApplication.clipboard()
        cb.clear()
        cb.setText(txt)


class ElementsDialogue(CenteredDialog):
    """
    Selected elements dialogue window
    """

    def __init__(self, name, elements: Union[List[ALL_DEV_TYPES], None] = None):
        super(ElementsDialogue, self).__init__()
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.layout = QtWidgets.QVBoxLayout(self)

        # build elements list
        self.objects_table = QtWidgets.QTableView()

        if elements is not None:
            if len(elements) > 0:
                model = ObjectsModel(objects=elements,
                                     time_index=None,
                                     property_list=list(elements[0].property_list),
                                     parent=self.objects_table,
                                     editable=False)

                self.objects_table.setModel(model)
            else:
                raise Exception("No elements passed :/")
        else:
            raise Exception("No elements passed :/")

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Proceed')
        self.accept_btn.clicked.connect(self.accept_click)

        # Copy button
        self.copy_btn = QtWidgets.QPushButton()
        self.copy_btn.setText('Copy')
        self.copy_btn.clicked.connect(self.copy_click)

        # add all to the GUI
        self.layout.addWidget(self.objects_table)
        self.frame2 = QtWidgets.QFrame()
        self.layout.addWidget(self.frame2)
        self.layout2 = QtWidgets.QHBoxLayout(self.frame2)

        self.layout2.addWidget(self.accept_btn)
        # self.layout2.addWidget(QtWidgets.QSpacerItem())
        self.layout2.addWidget(self.copy_btn)

        self.setLayout(self.layout)

        self.setWindowTitle(name)

        self.is_accepted = False

    def accept_click(self):
        """
        Accept action
        :return:
        """
        self.is_accepted = True
        self.accept()

    def copy_click(self):
        pass


class TimeReIndexDialogue(CenteredDialog):
    """
    New profile dialogue window
    """

    def __init__(self):
        super(TimeReIndexDialogue, self).__init__()
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted = False

        # year
        d2 = datetime.now()
        self.date_time_editor = QtWidgets.QDateTimeEdit()
        self.date_time_editor.setDateTime(QtCore.QDateTime(d2.year, d2.month, d2.day, d2.hour, d2.minute, d2.second))

        # time step length
        self.step_length = QtWidgets.QDoubleSpinBox()
        self.step_length.setMinimum(0.0001)
        self.step_length.setMaximum(1000)
        self.step_length.setValue(1)

        # units combo box
        self.units = QtWidgets.QComboBox()
        self.units.setModel(get_list_model(['h', 'm', 's']))

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # add all to the GUI
        self.main_layout.addWidget(QtWidgets.QLabel("Start date"))
        self.main_layout.addWidget(self.date_time_editor)

        self.main_layout.addWidget(QtWidgets.QLabel("Time step length"))
        self.main_layout.addWidget(self.step_length)

        self.main_layout.addWidget(QtWidgets.QLabel("Time units"))
        self.main_layout.addWidget(self.units)

        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle('Time re-index')

        h = 120
        self.resize(h, int(1.1 * h))

    def accept_click(self):
        """
        Accept and close
        """
        self.is_accepted = True
        self.accept()


class CorrectInconsistenciesDialogue(CenteredDialog):
    """
    New profile dialogue window
    """

    def __init__(self):
        super(CorrectInconsistenciesDialogue, self).__init__()
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted = False

        self.label1 = QtWidgets.QLabel()
        self.label1.setText("Minimum generator set point")

        # min voltage
        self.min_voltage = QtWidgets.QDoubleSpinBox()
        self.min_voltage.setMinimum(0)
        self.min_voltage.setMaximum(2)
        self.min_voltage.setSingleStep(0.01)
        self.min_voltage.setValue(0.98)

        self.label2 = QtWidgets.QLabel()
        self.label2.setText("Maximum generator set point")

        # min voltage
        self.max_voltage = QtWidgets.QDoubleSpinBox()
        self.max_voltage.setMinimum(0)
        self.max_voltage.setMaximum(2)
        self.max_voltage.setSingleStep(0.01)
        self.max_voltage.setValue(1.02)

        self.label3 = QtWidgets.QLabel()
        self.label3.setText("Maximum virtual tap difference")

        self.max_virtual_tap = QtWidgets.QDoubleSpinBox()
        self.max_virtual_tap.setMinimum(0)
        self.max_virtual_tap.setMaximum(1)
        self.max_virtual_tap.setSingleStep(0.01)
        self.max_virtual_tap.setValue(0.1)

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # add all to the GUI
        self.main_layout.addWidget(self.label1)
        self.main_layout.addWidget(self.min_voltage)
        self.main_layout.addWidget(self.label2)
        self.main_layout.addWidget(self.max_voltage)
        self.main_layout.addWidget(self.label3)
        self.main_layout.addWidget(self.max_virtual_tap)
        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle('Correct inconsistencies')

        h = 120
        self.resize(h, int(1.1 * h))

    def accept_click(self):
        """
        Accept and close
        """
        self.is_accepted = True
        self.accept()


def clear_qt_layout(layout):
    """
    Remove all widgets from a layout object
    :param layout:
    """
    for i in reversed(range(layout.count())):
        layout.itemAt(i).widget().deleteLater()


class CheckListDialogue(CenteredDialog):
    """
    New profile dialogue window
    """

    def __init__(self,
                 objects_list: List[str],
                 title='Select objects',
                 ask_for_group_name: bool = False,
                 group_label: str = "",
                 group_text: str = "",
                 default: List[bool] | None = None,
                 default_val: bool = True):
        """

        :param objects_list: List of names to display
        :param title: Window title
        :param ask_for_group_name: Ask for a group name (i.e. investments group name...)
        :param group_label: Name of the property
        :param group_text: Tentative group name
        """
        CenteredDialog.__init__(self)

        self.objects_list = objects_list
        self.default_vals = default if default is not None else [default_val for e in objects_list]

        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted: bool = False
        self.selected_indices: List[int] = list()
        self.status_dict: Dict[str, bool] = dict()

        self.label1 = QtWidgets.QLabel()
        self.label1.setText("Selected objects")

        self.group_label = QtWidgets.QLabel()
        self.group_label.setText(group_label)
        self.group_name_text = QtWidgets.QTextEdit()
        self.group_name_text.setText(group_text)
        self.group_name_text.setMaximumHeight(30)

        # list
        self.list_view = QtWidgets.QListView()
        # self.mdl = get_list_model(lst=objects_list,
        #                           checks=True,
        #                           check_value=default_val)

        self.mdl = get_chck_list_model(
            lst=objects_list,
            check_status=self.default_vals
        )

        self.list_view.setModel(self.mdl)

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # add all to the GUI
        if ask_for_group_name:
            self.main_layout.addWidget(self.group_label)
            self.main_layout.addWidget(self.group_name_text)

        self.main_layout.addWidget(self.label1)
        self.main_layout.addWidget(self.list_view)
        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle(title)

        h = 260
        self.resize(h, int(0.8 * h))

    def get_group_text(self) -> str:
        """
        Get the group text
        :return: string
        """
        return self.group_name_text.toPlainText()

    def selected(self, val: str) -> bool:
        """
        Check if the value was selected
        :param val:
        :return:
        """
        return self.status_dict.get(val, False)

    def accept_click(self):
        """
        Accept and close
        """
        self.is_accepted = True

        self.selected_indices: IntVec = get_checked_indices(self.mdl)

        for row in range(self.mdl.rowCount()):
            item = self.mdl.item(row)
            if item.checkState() == QtCore.Qt.CheckState.Checked:
                self.status_dict[self.objects_list[row]] = True
            else:
                self.status_dict[self.objects_list[row]] = False

        self.accept()


class DeleteDialogue(CenteredDialog):
    """
    New profile dialogue window
    """

    def __init__(self,
                 names_list: List[str],
                 title='Select objects',
                 delete_from_db: bool = False,
                 ask_for_group_name: bool = False,
                 group_label: str = "",
                 group_text: str = "",
                 checks=True,
                 check_value=True):
        """

        :param names_list: List of names to display
        :param title: Window title
        :param ask_for_group_name: Ask for a group name (i.e. investments group name...)
        :param group_label: Name of the property
        :param group_text: Tentative group name
        """
        CenteredDialog.__init__(self)
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted: bool = False
        self.selected_indices: List[int] = list()

        self.label1 = QtWidgets.QLabel()
        self.label1.setText("Selected objects")

        self.group_label = QtWidgets.QLabel()
        self.group_label.setText(group_label)
        self.group_name_text = QtWidgets.QTextEdit()
        self.group_name_text.setText(group_text)
        self.group_name_text.setMaximumHeight(30)

        # list
        self.list_view = QtWidgets.QListView()
        self.mdl = get_list_model(names_list, checks=checks, check_value=check_value)
        self.list_view.setModel(self.mdl)

        # delete_with_dialogue from DB check
        self.delete_from_db_check = QtWidgets.QCheckBox()
        self.delete_from_db_check.setText(r'Remove from the database')
        self.delete_from_db_check.setChecked(delete_from_db)

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # add all to the GUI
        if ask_for_group_name:
            self.main_layout.addWidget(self.group_label)
            self.main_layout.addWidget(self.group_name_text)

        self.main_layout.addWidget(self.label1)
        self.main_layout.addWidget(self.list_view)
        self.main_layout.addWidget(self.delete_from_db_check)
        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle(title)

        h = 260
        self.resize(h, int(0.8 * h))

    @property
    def delete_from_db(self):
        return self.delete_from_db_check.isChecked()

    def get_group_text(self) -> str:
        """
        Get the group text
        :return: string
        """
        return self.group_name_text.toPlainText()

    def accept_click(self):
        """
        Accept and close
        """
        self.is_accepted = True

        self.selected_indices: IntVec = get_checked_indices(self.mdl)
        self.accept()


class InputNumberDialogue(CenteredDialog):
    """
    New InputNumberDialogue window
    """

    def __init__(self, min_value: float, max_value: float, default_value: float, is_int: bool = False,
                 title='Select objects', text='', decimals=2, suffix='', h=80, w=240):
        """

        :param min_value:
        :param max_value:
        :param is_int:
        :param title:
        :param text:
        :param decimals:
        :param suffix:
        :param h:
        :param w:
        """
        CenteredDialog.__init__(self)
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted: bool = False
        self.value = 0 if is_int else 0.0

        self.label1 = QtWidgets.QLabel()
        self.label1.setText(text)

        # min voltage
        self.input_box = QtWidgets.QSpinBox() if is_int else QtWidgets.QDoubleSpinBox()
        self.input_box.setMinimum(min_value)
        self.input_box.setMaximum(max_value)
        self.input_box.setSuffix(suffix)
        self.input_box.setValue(default_value)

        if not is_int:
            self.input_box.setDecimals(decimals)

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # add all to the GUI
        self.main_layout.addWidget(self.label1)
        self.main_layout.addWidget(self.input_box)
        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle(title)

        self.resize(w, h)

    def accept_click(self):
        """
        Accept and close
        """
        self.is_accepted = True

        self.value = self.input_box.value()
        self.accept()


class InputSearchDialogue(CenteredDialog):
    """
    New InputNumberDialogue window
    """

    def __init__(self, deafault_value: str, title='Search', prompt='', h=80, w=240):
        """
        :default_value:
        :param title:
        :param prompt:
        :param h:
        :param w:
        """

        self.searchText = ""
        CenteredDialog.__init__(self)
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted: bool = False

        self.label1 = QtWidgets.QLabel()
        self.label1.setText(prompt)

        # min voltage
        self.input_box = QtWidgets.QLineEdit()

        # search button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Search')
        self.accept_btn.clicked.connect(self.search_click)

        # add all to the GUI
        self.main_layout.addWidget(self.label1)
        self.main_layout.addWidget(self.input_box)
        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle(title)

        self.resize(w, h)

    def search_click(self):
        """
        Serach and close
        """
        self.is_accepted = True

        self.searchText = self.input_box.text()
        self.accept()


class StartEndSelectionDialogue(CenteredDialog):
    """
    New StartEndSelectionDialogue window
    """

    def __init__(self, min_value: int, max_value: int, time_array,
                 title='Simulation limits selection', h=80, w=240):
        """

        :param min_value:
        :param max_value:
        :param time_array:
        :param title:
        :param h:
        :param w:
        """
        CenteredDialog.__init__(self)
        self.setObjectName("self")
        self.setContextMenuPolicy(QtCore.Qt.ContextMenuPolicy.NoContextMenu)
        self.main_layout = QtWidgets.QVBoxLayout(self)

        self.is_accepted: bool = False

        self.time_array = time_array
        nt = len(time_array) - 1

        self.start_slider = QtWidgets.QSlider()
        self.start_slider.setMinimum(0)
        self.start_slider.setMaximum(nt)
        self.start_slider.setValue(min_value)
        self.start_slider.valueChanged.connect(self.slider_change)
        self.start_slider.setOrientation(QtCore.Qt.Orientation.Horizontal)

        self.start_label = QtWidgets.QLabel()

        self.end_slider = QtWidgets.QSlider()
        self.end_slider.setMinimum(0)
        self.end_slider.setMaximum(nt)
        self.end_slider.setValue(max_value)
        self.end_slider.valueChanged.connect(self.slider_change)
        self.end_slider.setOrientation(QtCore.Qt.Orientation.Horizontal)

        self.end_label = QtWidgets.QLabel()

        # accept button
        self.accept_btn = QtWidgets.QPushButton()
        self.accept_btn.setText('Accept')
        self.accept_btn.clicked.connect(self.accept_click)

        # add all to the GUI
        self.main_layout.addWidget(self.start_slider)
        self.main_layout.addWidget(self.start_label)
        self.main_layout.addWidget(self.end_slider)
        self.main_layout.addWidget(self.end_label)
        self.main_layout.addWidget(self.accept_btn)

        self.setLayout(self.main_layout)

        self.setWindowTitle(title)

        self.start_value = min_value
        self.end_value = max_value

        self.resize(w, h)

        self.slider_change()

    def slider_change(self):
        """
        On any slider change...
        """
        self.start_value = self.start_slider.value()
        self.end_value = self.end_slider.value()

        if self.start_value > self.end_value:
            self.end_slider.setValue(self.start_value)
            self.end_value = self.start_value

        t1 = pd.to_datetime(self.time_array[self.start_value]).strftime('%d/%m/%Y %H:%M')
        t2 = pd.to_datetime(self.time_array[self.end_value]).strftime('%d/%m/%Y %H:%M')
        self.start_label.setText(str(t1))
        self.end_label.setText(str(t2) + ' [{0}]'.format(self.end_value - self.start_value))

    def accept_click(self):
        """
        Accept and close
        """
        self.is_accepted = True

        self.accept()


class CustomQuestionDialogue(CenteredDialog):
    """
    Custom question dialogue
    """

    def __init__(self, title: str, question: str, answer1: str, answer2: str):
        super().__init__()

        self.setWindowTitle(title)

        layout = QtWidgets.QVBoxLayout()
        button_layout = QtWidgets.QHBoxLayout()

        label = QtWidgets.QLabel(question)
        label.setWordWrap(True)
        layout.addWidget(label)

        button_layout.addSpacerItem(QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Policy.Expanding))

        button_1 = QtWidgets.QPushButton(answer1)
        button_1.clicked.connect(self.b1_clicked)
        button_layout.addWidget(button_1)

        button_2 = QtWidgets.QPushButton(answer2)
        button_2.clicked.connect(self.b2_clicked)
        button_layout.addWidget(button_2)

        layout.addLayout(button_layout)
        self.setLayout(layout)

        self.accepted_answer = 0

    def b1_clicked(self):
        """
        Button 1 clicked
        :return:
        """
        self.accepted_answer = 1
        self.accept()

    def b2_clicked(self):
        """
        Button 2 clicked
        :return:
        """
        self.accepted_answer = 2
        self.accept()


class ArrayTableModel(QAbstractTableModel):
    """
    ArrayTableModel
    """

    def __init__(self, data: List[np.ndarray], headers: List[str]):
        super().__init__()

        self._data = data
        self.headers = headers

    def get_data(self) -> List[np.ndarray]:
        """
        Get the model internal data
        :return: list of arrays
        """
        return self._data

    def rowCount(self, parent: QModelIndex = QModelIndex()) -> int:
        """

        :param parent:
        :return:
        """
        return self._data[0].shape[0]

    def columnCount(self, parent: QModelIndex = QModelIndex()):
        """

        :param parent:
        :return:
        """
        return len(self._data)  # We have two columns, one for each array

    def headerData(self, section: int, orientation: QtCore.Qt.Orientation,
                   role: int = Qt.ItemDataRole.DisplayRole) -> Any:
        """

        :param section:
        :param orientation:
        :param role:
        :return:
        """
        if role == Qt.ItemDataRole.DisplayRole:
            if orientation == Qt.Orientation.Horizontal:
                return self.headers[section]
            if orientation == Qt.Orientation.Vertical:
                return section  # To show row numbers starting from 0
        return None

    def data(self, index: QModelIndex, role: int = Qt.ItemDataRole.DisplayRole) -> Union[None, str]:
        """

        :param index:
        :param role:
        :return:
        """
        if not index.isValid():
            return None

        if role == Qt.ItemDataRole.DisplayRole:
            row = index.row()
            column = index.column()
            return str(self._data[column][row])

        return None

    def setData(self, index: QModelIndex, value: float, role=Qt.ItemDataRole.EditRole):
        """

        :param index:
        :param value:
        :param role:
        :return:
        """
        if not index.isValid():
            return False

        if role == Qt.ItemDataRole.EditRole:
            row = index.row()
            column = index.column()
            try:
                value = float(value)
            except ValueError:
                return False

            self._data[column][row] = value

            self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole])
            return True

        return False

    def flags(self, index: QModelIndex) -> Qt.ItemFlag:
        """

        :param index:
        :return:
        """
        if not index.isValid():
            return Qt.ItemFlag.NoItemFlags
        return Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsEditable

    def insertRows(self, position: int, rows=1, parent=QModelIndex()):
        """

        :param position:
        :param rows:
        :param parent:
        :return:
        """
        self.beginInsertRows(parent, position, position + rows - 1)

        for i in range(len(self._data)):
            self._data[i] = np.insert(self._data[i], position, [0] * rows)

        # set active to true
        self._data[0][-1] = 1

        self.endInsertRows()
        return True

    def removeRows(self, position: int, rows=1, parent=QModelIndex()):
        """

        :param position:
        :param rows:
        :param parent:
        :return:
        """
        self.beginRemoveRows(parent, position, position + rows - 1)

        for i in range(len(self._data)):
            self._data[i] = np.delete(self._data[i], slice(position, position + rows))

        self.endRemoveRows()
        return True


class ArrayEditor(CenteredDialog):
    """
    ArrayEditor
    """

    def __init__(self):
        CenteredDialog.__init__(self)

        self.setWindowTitle("Array Editor")

        self._g_steps = np.arange(10)
        self._b_steps = np.arange(10)
        self.model = ArrayTableModel(data=[self._g_steps, self._b_steps], headers=["G", "B"])

        self.table_view = QTableView()
        self.table_view.setModel(self.model)

        self.add_button = QPushButton("Add")
        self.delete_button = QPushButton("Delete")

        self.add_button.clicked.connect(self.add_row)
        self.delete_button.clicked.connect(self.delete_row)

        layout = QVBoxLayout()
        layout.addWidget(self.table_view)

        button_layout = QHBoxLayout()
        button_layout.addWidget(self.add_button)
        button_layout.addWidget(self.delete_button)

        layout.addLayout(button_layout)
        self.setLayout(layout)

    def add_row(self):
        """
        Add row
        """
        row_count = self.model.rowCount()
        self.model.insertRows(row_count, 1)

    def delete_row(self):
        """
        Delete the selected rows
        """
        selected_indexes = self.table_view.selectionModel().selectedIndexes()

        rows = list({index.row() for index in selected_indexes})
        rows.sort(reverse=True)
        for r in rows:
            self.model.removeRows(position=r, rows=1)


# ----------------------------------------------------
# VALIDITY RULES
# ----------------------------------------------------


def valid_methods_for_fault(fault: FaultType):
    """
    :param fault:
    :return:
    """
    # all other faults allow both
    return list(MethodShortCircuit)


def valid_phases_for_fault(fault: FaultType):
    """

    :param fault:
    :return:
    """
    if fault in (FaultType.LLLG, FaultType.LLL):
        return [PhasesShortCircuit.abc]

    if fault == FaultType.LG:
        return [PhasesShortCircuit.a, PhasesShortCircuit.b, PhasesShortCircuit.c]

    if fault in (FaultType.LL, FaultType.LLG):
        return [PhasesShortCircuit.ab, PhasesShortCircuit.bc, PhasesShortCircuit.ca]

    return []


class ShortCircuitSelector(CenteredDialog):
    """
    ShortCircuitSelector
    """

    def __init__(self) -> None:
        super().__init__()
        self.setWindowTitle("Short Circuit Configuration")

        layout = QVBoxLayout(self)

        # Fault type
        self.cb_fault = QComboBox()
        self.cb_fault.addItems([e.value for e in FaultType])
        layout.addWidget(QLabel("Fault type:"))
        layout.addWidget(self.cb_fault)

        # Method
        self.cb_method = QComboBox()
        self.cb_method.addItems([e.value for e in MethodShortCircuit])
        layout.addWidget(QLabel("Method:"))
        layout.addWidget(self.cb_method)

        # Phases
        self.cb_phases = QComboBox()
        self.cb_phases.addItems([e.value for e in PhasesShortCircuit])
        self.phases_label = QLabel("Phases:")
        layout.addWidget(self.phases_label)
        layout.addWidget(self.cb_phases)

        # --- VERTICAL SPACER ---
        layout.addSpacerItem(QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding))

        # Accept button
        self.btn_accept = QPushButton("Accept")
        layout.addWidget(self.btn_accept)

        # Logic connections
        self.cb_fault.currentIndexChanged.connect(self.update_logic)
        self.btn_accept.clicked.connect(self.accept_clicked)

        self.update_logic()
        self.fault = FaultType(self.cb_fault.currentText())
        self.method = MethodShortCircuit(self.cb_method.currentText())
        self.phases = PhasesShortCircuit(self.cb_phases.currentText())

        self.was_accepted = False

        self.cb_method.currentTextChanged.connect(self.update_view)

    def update_view(self):
        """

        :return:
        """

        fault = FaultType(self.cb_fault.currentText())

        # -------- UPDATE PHASES --------
        if self.cb_method.currentText() == MethodShortCircuit.sequences.value:
            self.cb_phases.setVisible(False)
            self.phases_label.setVisible(False)
        else:
            self.cb_phases.setVisible(True)
            self.phases_label.setVisible(True)
            allowed_phases = valid_phases_for_fault(fault)
            current_phase = self.cb_phases.currentText()

            self.cb_phases.clear()
            for p in allowed_phases:
                self.cb_phases.addItem(p.value)

            if current_phase in [p.value for p in allowed_phases]:
                self.cb_phases.setCurrentText(current_phase)

    def update_logic(self):
        """Update available method and phase options based on the fault type."""

        fault = FaultType(self.cb_fault.currentText())

        # -------- UPDATE METHOD --------
        allowed_methods = valid_methods_for_fault(fault)
        current_method = self.cb_method.currentText()

        self.cb_method.clear()
        for m in allowed_methods:
            self.cb_method.addItem(m.value)

        if current_method in [m.value for m in allowed_methods]:
            self.cb_method.setCurrentText(current_method)

        # -------- UPDATE PHASES --------
        if current_method == MethodShortCircuit.sequences.value:
            self.cb_phases.setVisible(False)
            self.phases_label.setVisible(False)
        else:
            self.cb_phases.setVisible(True)
            self.phases_label.setVisible(True)
            allowed_phases = valid_phases_for_fault(fault)
            current_phase = self.cb_phases.currentText()

            self.cb_phases.clear()
            for p in allowed_phases:
                self.cb_phases.addItem(p.value)

            if current_phase in [p.value for p in allowed_phases]:
                self.cb_phases.setCurrentText(current_phase)

        self.fault = FaultType(self.cb_fault.currentText())
        self.method = MethodShortCircuit(self.cb_method.currentText())
        self.phases = PhasesShortCircuit(self.cb_phases.currentText())

    def get_selection(self):
        """Return the selected configuration as enums."""
        return (
            FaultType(self.cb_fault.currentText()),
            MethodShortCircuit(self.cb_method.currentText()),
            PhasesShortCircuit(self.cb_phases.currentText()),
        )

    def accept_clicked(self):
        """Check if values are valid and close dialog."""
        self.fault = FaultType(self.cb_fault.currentText())
        self.method = MethodShortCircuit(self.cb_method.currentText())
        self.phases = PhasesShortCircuit(self.cb_phases.currentText())
        self.was_accepted = True
        self.close()


class FileTypeSelector(CenteredDialog):
    """
    FileTypeSelector
    """

    def __init__(self, file_name: List[str] | str) -> None:
        super().__init__()
        self.setWindowTitle("Select how to load the file")
        self.setModal(False)
        layout = QVBoxLayout(self)

        if isinstance(file_name, list):
            txt = "You've passed a generic list of files\nselect the expected processing format"

            xml_types_count = 0
            ucte_types_count = 0
            for f in file_name:
                if f.endswith(".xml"):
                    xml_types_count += 1
                elif f.endswith(".zip"):
                    xml_types_count += 1
                elif f.endswith(".uct"):
                    ucte_types_count += 1
                elif f.endswith(".ucte"):
                    ucte_types_count += 1

            if xml_types_count > 0:
                tpes = [FileType.CGMES, FileType.CIM, FileType.Iidm]
            elif ucte_types_count > 0:
                tpes = [FileType.UCTE]
            else:
                tpes = []

        elif isinstance(file_name, str):
            txt = "You've passed a generic of file\nselect the expected processing format"

            if file_name.endswith(".xml"):
                tpes = [FileType.CGMES, FileType.CIM, FileType.Iidm]
            elif file_name.endswith(".zip"):
                tpes = [FileType.CGMES, FileType.CIM, FileType.Iidm]
            elif file_name.endswith(".uct"):
                tpes = [FileType.UCTE]
            elif file_name.endswith(".ucte"):
                tpes = [FileType.UCTE]
            else:
                tpes = []

        else:
            raise ValueError("Files should be a list of a string")

        # Text
        layout.addWidget(QLabel(txt))

        # Method
        self.tpe_dict = {e.value: e for e in tpes}
        self.cb_method = QComboBox()
        self.cb_method.addItems([e.value for e in tpes])
        layout.addWidget(QLabel("Format:"))
        layout.addWidget(self.cb_method)

        # --- VERTICAL SPACER ---
        layout.addSpacerItem(QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding))

        # Accept button
        self.btn_accept = QPushButton("Accept")
        layout.addWidget(self.btn_accept)

        # Logic connections
        self.btn_accept.clicked.connect(self.accept_clicked)

        self.was_accepted = False
        self.file_type: FileType | None = None

    def accept_clicked(self):
        """Check if values are valid and close dialog."""
        self.file_type = self.tpe_dict[self.cb_method.currentText()]
        self.was_accepted = True
        self.close()


class CgmesOptionsSelector(CenteredDialog):
    """
    FileTypeSelector
    """

    def __init__(self) -> None:
        super().__init__()
        self.setWindowTitle("Select the CGMES options")
        self.setModal(False)
        layout = QVBoxLayout(self)

        tpes = [CGMESVersions.v2_4_15, CGMESVersions.v3_0_0]

        # Method
        self.tpe_dict = {e.value: e for e in tpes}
        self.cb_method = QComboBox()
        self.cb_method.addItems([e.value for e in tpes])
        layout.addWidget(QLabel("CGMES Version:"))
        layout.addWidget(self.cb_method)

        # --- VERTICAL SPACER ---
        layout.addSpacerItem(QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding))

        # Accept button
        self.btn_accept = QPushButton("Accept")
        layout.addWidget(self.btn_accept)

        # Logic connections
        self.btn_accept.clicked.connect(self.accept_clicked)

        self.was_accepted = False
        self.version: CGMESVersions | None = None

    def accept_clicked(self):
        """Check if values are valid and close dialog."""
        self.version = self.tpe_dict[self.cb_method.currentText()]
        self.was_accepted = True
        self.close()


if __name__ == "__main__":
    import sys

    app = QApplication(sys.argv)
    w = ShortCircuitSelector()
    w.show()
    sys.exit(app.exec())

    # from PySide6.QtWidgets import QApplication
    #
    # app = QApplication(sys.argv)
    # # window = InputNumberDialogue(min_value=3,
    # #                              max_value=10,
    # #                              default_value=3,
    # #                              is_int=True,
    # #                              title="stuff",
    # #                              text="valor? fsd..xcfh.dfgbhdfbflb.lsdfnblsndf.bnsdf.bn.xdfnb.xdfbñlxdhfn.blxnd",
    # #                              suffix=' cosas')
    #
    # window = CustomQuestionDialogue(title="My question",
    #                                 question="What do you want " * 10,
    #                                 answer1="Go home",
    #                                 answer2="stay here")
    #
    # window.show()
    # sys.exit(app.exec())

    app = QApplication(sys.argv)
    window = ArrayEditor()
    window.resize(400, 300)
    window.show()
    sys.exit(app.exec())
